      module rotation
      contains
!      program rotation
!      integer :: i,n
!      character(len=30) :: input,output
!      character line
!      character,dimension(:),allocatable :: atom
!      real(kind=8) :: phi
!      real(kind=8),dimension(3) :: x,axis
!      real(kind=8),dimension(4) :: rotq
!      real(kind=8),dimension(:,:),allocatable :: coord,newcoord
!      print *, "type input filename (max 30 characters)"
!      read (*,*) input
!      print *, "type output filename (max 30 characters)"
!      read (*,*) output
!      print *, "type rotation origin coordinates"
!      read (*,*) (x(i),i=1,3)
!      print *, "type rotation axis coordinates"
!      read (*,*) (axis(i),i=1,3)
!      print *, "x=",x(1),"y=",x(2),"z=",x(3)
!      print *, "type rotation angle in degrees"
!      read (*,*) phi
!      open(1,file=input)
!      open(2,file=output)
!      n=0
!      do
!         read (1,*,end=10,err=10) line
!         n=n+1
!         cycle
!10       exit
!      enddo
!      allocate (atom(n),coord(3,n),newcoord(3,n))
!      rewind 1
!      do i=1,n
!         read (1,'(a,2x,3f8.3)') atom(i),(coord(j,i),j=1,3)
!      enddo
!      call rotquat(axis,phi,rotq)
!      print *, "q=",rotq(1),"+",rotq(2),"i+",rotq(3),"j+",rotq(4),"k"
!      call grouprot(x,axis,phi,coord,newcoord,n)
!      do i=1,n
!         write (2,*) atom(i),(newcoord(j,i),j=1,3)
!      enddo
!      call vecrot(x,rotq,y)
!      print *, "new vector coordinates",(y(i),i=1,3)
!      end program
!----------------------------------------------------------------
      subroutine rotquat(axis,angle,rotq)
!
!   Create a rotation quaternion for given axis and angle (in radians)
!
      real(kind=8),dimension(3) :: axis
      real(kind=8) :: angle,axnorm,imfac,pi,deg2rad
      real(kind=8),dimension(4) :: rotq
!      pi=4.0D0*atan(1.0D0)
!      deg2rad=pi/180.0D0 
!      print *, "pi=",pi 
      axnorm=1/sqrt(axis(1)**2+axis(2)**2+axis(3)**2)
      do i=1,3
         axis(i)=axis(i)*axnorm
      enddo
      angle=0.5*angle
      rotq(1)=cos(angle)
      imfac=sin(angle)
      do i=1,3
         rotq(i+1)=imfac*axis(i)
      enddo
      end subroutine rotquat
!----------------------------------------------------------------
      subroutine multquat(q,r,qr)
!
!   multiply quaternions q and r to get qr
!
      real(kind=8),dimension(4) :: q,r,qr
      qr(1)=q(1)*r(1)-q(2)*r(2)-q(3)*r(3)-q(4)*r(4)
      qr(2)=q(1)*r(2)+q(2)*r(1)+q(3)*r(4)-q(4)*r(3)
      qr(3)=q(1)*r(3)+q(3)*r(1)-q(2)*r(4)+q(4)*r(2)
      qr(4)=q(1)*r(4)+q(4)*r(1)+q(2)*r(3)-q(3)*r(2)
      end subroutine multquat
!----------------------------------------------------------------
      subroutine vecrot(vec,rotq,newvec)
!
!   rotate vector vec using rotation quaternion rotq
! 
      real(kind=8),dimension(3) :: vec,newvec
      real(kind=8),dimension(4) :: rotq,invrotq,q,vecq,newvecq
      vecq(1)=0
      do i=1,3
         vecq(i+1)=vec(i)
      enddo
      invrotq(1)=rotq(1)
      do i=2,4
         invrotq(i)=-rotq(i)
      enddo
      call multquat(rotq,vecq,q)
      call multquat(q,invrotq,newvecq)
      do i=1,3
         newvec(i)=newvecq(i+1)
      enddo
      end subroutine vecrot

!----------------------------------------------------------------
      subroutine grouprot(rotorig,rotq,coord,newcoord,n)
!      subroutine grouprot(rotorig,axis,angle,coord,newcoord,n)
!
!   rotate a group of coordinates coord around axis set at origin point rotorig by angle (in degrees)
!
      integer :: n
      real(kind=8) :: angle
      real(kind=8),dimension(3) :: rotorig,axis,x,y
      real(kind=8),dimension(4) :: rotq
      real(kind=8),dimension(3,n) :: coord,newcoord
!      call rotquat(axis,angle,rotq)
!      print *, "q=",rotq(1),"+",rotq(2),"i+",rotq(3),"j+",rotq(4),"k"
      do i=1,n
         do j=1,3
            x(j)=coord(j,i)-rotorig(j)
         enddo
         call vecrot(x,rotq,y)
         do j=1,3
            newcoord(j,i)=y(j)+rotorig(j)
         enddo
      enddo
      end subroutine grouprot
!----------------------------------------------------------------

!----------------------------------------------------------------
      subroutine findrot(refx,refy,rotq)
!
!   find rotation quaternion for transformation from xyz to an orthogonal base specified by Gram-Schmidt orthogonalization of newx and newy
!
      use io_units
      real(kind=8) :: alpha,beta,normx,normy,xy
      real(kind=8),dimension(3) :: refx,refy,oldx,oldy,newx,newy,y,axis, &
                                   by
      real(kind=8),dimension(4) :: rotq,q1,q2,invq1
      data oldy/0,1,0/,oldx/1,0,0/
!      pi=4.0D0*atan(1.0D0)
!      rad2deg=180.0D0/pi
      normx=1/sqrt(refx(1)**2+refx(2)**2+refx(3)**2)
!      print *, normx
      xy=0
      do i=1,3
         refx(i)=refx(i)*normx
         xy=xy+refy(i)*refx(i)
      enddo
!      print *, newx
      normy=0
      do i=1,3
         refy(i)=refy(i)-refx(i)*xy
         normy=normy+refy(i)**2
      enddo
      normy=1/sqrt(normy)
      do i=1,3
        refy(i)=refy(i)*normy
      enddo
!      print *, newy
      alpha=acos(refx(1))
!      print *, alpha
      axis(1)=0
      axis(2)=-refx(3)
      axis(3)=refx(2)
      call rotquat(axis,alpha,q1)
      call vecrot(oldy,q1,y)
      invq1(1)=q1(1)
      do i=2,4
         invq1(i)=-q1(i)
      enddo
      call vecrot(refy,invq1,by)
      beta=atan2(by(3),by(2))
      call rotquat(refx,beta,q2)
      call multquat(q2,q1,rotq)
!----------------------------------------------------------------
!   just for debugging
!----------------------------------------------------------------
      call vecrot(oldx,rotq,newx)
      write(iout,*) "ref. x = ", refx
      write(iout,*) " new x = ", newx
      call vecrot(oldy,rotq,newy)
      write(iout,*) "ref. y = ", refy
      write(iout,*) " new y = ", newy
!----------------------------------------------------------------
      end subroutine findrot

!----------------------------------------------------------------
      end module rotation
